import typing

from yuque_py.clients.abstract_client import AbstractClient


class User:
    def __init__(self, client: AbstractClient):
        self._client = client

    def get(self, login: typing.Optional[str] = None) -> typing.Dict:
        api = f"users/{login}" if login else "user"
        return self._client.request(api=api, method="GET", requests_data=None)



import typing

from yuque_py.clients.abstract_client import AbstractClient


class Group:
    def __init__(self, client: AbstractClient):
        self._client = client

    def list(
        self,
        login: typing.Optional[str] = None,
        data: typing.Optional[typing.Dict] = None,
    ) -> typing.Dict:
        api = f"users/{login}/groups" if login else "groups"
        return self._client.request(api, method="GET", requests_data=data)

    def create(self, data: typing.Dict) -> typing.Dict:
        assert "name" in data
        assert "login" in data
        api = "groups"
        return self._client.request(api, method="POST", requests_data=data)

    def get(self, login: str) -> typing.Dict:
        assert login
        api = f"groups/{login}"
        return self._client.request(api, method="GET")

    def update(self, login: str, data: typing.Dict) -> typing.Dict:
        assert login
        api = f"groups/{login}"
        return self._client.request(api, method="PUT", requests_data=data)

    def delete(self, login: str) -> typing.Dict:
        assert login
        api = f"groups/{login}"
        return self._client.request(api, method="DELETE")

    def list_user(self, login: str) -> typing.Dict:
        return self._client.request(f"groups/{login}/users", method="GET")

    def add_user(self, group: str, user: str, data: typing.Dict) -> typing.Dict:
        assert group
        assert user
        api = f"groups/{group}/users/{user}"
        return self._client.request(api, method="PUT", requests_data=data)

    def remove_user(self, group: str, user: str) -> typing.Dict:
        assert group
        assert user
        api = f"groups/{group}/users/{user}"
        return self._client.request(api, method="DELETE")

    @staticmethod
    def _get_url(user: str, group: str):
        assert user or group
        api = f"users/{user}/repos" if user else f"users/{group}/repos"
        return api



import typing

import requests
from urllib.parse import urlencode

from .abstract_client import AbstractClient
from yuque_py.exceptions.request_error import RequestError


class Client(AbstractClient):
    api_host: str
    user_token: str

    def __init__(self, api_host: str, user_token: str) -> None:
        self.api_host = api_host
        self.user_token = user_token

    def request(
        self,
        api: str,
        method: str,
        requests_data: typing.Optional[typing.Dict[str, typing.Any]] = None,
        user_agent: str = "@yuque/sdk",
    ) -> typing.Dict:
        request_url = f"{self.api_host}/{api}"
        request_header = {"User-Agent": user_agent, "X-Auth-Token": self.user_token}
        if method == "GET":
            func = self._get_request
        elif method == "POST":
            request_header["Content-Type"] = "application/json"
            func = self._post_request
        elif method == "PUT":
            request_header["Content-Type"] = "application/json"
            func = self._put_request
        elif method == "DELETE":
            func = self._delete_request
        else:
            raise ValueError
        response = func(request_url, requests_data, request_header)
        if response.status_code != 200:
            raise RequestError(response.status_code, response.text)
        return response.json()

    @staticmethod
    def _get_request(
        request_url: str,
        requests_data: typing.Dict[str, typing.Any],
        request_header: typing.Dict[str, str],
    ) -> requests.Response:
        if requests_data:
            request_url = f"{request_url}&{urlencode(requests_data)}"

        return requests.get(request_url, headers=request_header)

    @staticmethod
    def _post_request(
        request_url: str,
        requests_data: typing.Dict[str, typing.Any],
        request_header: typing.Dict[str, str],
    ) -> requests.Response:
        return requests.post(request_url, json=requests_data, headers=request_header)

    @staticmethod
    def _put_request(
        request_url: str,
        requests_data: typing.Dict[str, typing.Any],
        request_header: typing.Dict[str, str],
    ) -> requests.Response:
        return requests.put(request_url, json=requests_data, headers=request_header)

    @staticmethod
    def _delete_request(
        request_url: str,
        requests_data: typing.Dict[str, typing.Any],
        request_header: typing.Dict[str, str],
    ) -> requests.Response:
        return requests.delete(request_url, headers=request_header)



import abc
import typing


class AbstractClient:
    @abc.abstractmethod
    def request(
        self,
        api: str,
        method: str,
        requests_data: typing.Optional[typing.Dict[str, typing.Any]] = None,
    ) -> typing.Dict:
        pass



from yuque_py.models import User, Group, Repo, Doc
from yuque_py.clients.client import Client


class Yuque:
    def __init__(self, api_host: str, user_token: str):
        self._client = Client(api_host, user_token)
        self.users = User(self._client)
        self.groups = Group(self._client)
        self.repos = Repo(self._client)
        self.docs = Doc(self._client)



import typing

from yuque_py.clients.abstract_client import AbstractClient


class Repo:
    def __init__(self, client: AbstractClient):
        self._client = client

    def list(
        self,
        user: typing.Optional[str] = None,
        group: typing.Optional[str] = None,
        data: typing.Optional[typing.Dict] = None,
    ) -> typing.Dict:
        api = self._get_url(user, group)
        return self._client.request(api, method="GET", requests_data=data)

    def create(
        self,
        user: typing.Optional[str] = None,
        group: typing.Optional[str] = None,
        data: typing.Optional[typing.Dict] = None,
    ) -> typing.Dict:
        api = self._get_url(user, group)
        assert data
        return self._client.request(api, method="POST", requests_data=data)

    def get(self, namespace: str, data: typing.Dict = None) -> typing.Dict:
        assert namespace
        return self._client.request(
            f"repos/{namespace}", method="GET", requests_data=data
        )

    def update(self, namespace: str, data: typing.Dict) -> typing.Dict:
        assert namespace
        return self._client.request(
            f"repos/{namespace}", method="PUT", requests_data=data
        )

    def delete(self, namespace: str) -> typing.Dict:
        assert namespace
        return self._client.request(f"repos/{namespace}", method="DELETE")

    @staticmethod
    def _get_url(user: str, group: str):
        assert user or group
        api = f"users/{user}/repos" if user else f"groups/{group}/repos"
        return api



class RequestError(Exception):
    def __init__(self, status_code: int, response_body: str):
        message = f"Request for yuque failed, response_code:{status_code},response_body:{response_body}"
        super().__init__(message)



import typing

from yuque_py.clients.abstract_client import AbstractClient


class Doc:
    def __init__(self, client: AbstractClient):
        self._client = client

    def get(
        self, namespace: str, slug: str, data: typing.Optional[typing.Dict] = None
    ) -> typing.Dict:
        assert namespace
        assert slug
        api = f"repos/{namespace}/docs/{slug}"
        return self._client.request(api, method="GET", requests_data=data)

    def create(
        self, namespace: str, data: typing.Optional[typing.Dict] = None
    ) -> typing.Dict:
        assert namespace
        api = f"repos/{namespace}/docs"
        return self._client.request(api, method="POST", requests_data=data)

    def update(
        self, namespace: str, doc_id: str, data: typing.Optional[typing.Dict] = None
    ) -> typing.Dict:
        assert namespace
        assert doc_id
        api = f"repos/{namespace}/docs/{doc_id}"
        return self._client.request(api, method="PUT", requests_data=data)

    def delete(self, namespace: str, doc_id: str) -> typing.Dict:
        assert namespace
        assert doc_id
        api = f"repos/{namespace}/docs/{doc_id}"
        return self._client.request(api, method="DELETE")



import os
from setuptools import setup
from yuque_py import __version__

here = os.path.abspath(os.path.dirname(__file__))

packages = ["yuque_py", "yuque_py.models", "yuque_py.clients", "yuque_py.exceptions"]

requires = [
    "pytest==5.1.1",
    "pytest-cov==2.7.1",
    "requests==2.22.0",
    "requests-mock==1.6.0"
]

setup(
    name="yuque-py",
    version=__version__,
    description="yuque api for python version",
    author="Manjusaka",
    author_email="me@manjusaka.me",
    url="https://github.com/Zheaoli/yueque-py",
    packages=packages,
    install_requires=requires,
    include_package_data=True,
    license="MIT License",
    python_requires=">=3.6",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
)
